import numpy as np
import scipy.signal as signal
import matplotlib.pyplot as plt

# This script shows how the filter design function
# can be used to find the coefficients of the biquads decomposition
# To this aim, the zpk (zeroes, poles, gain) output is chosen
# for the filter design formula

# This script refer to a butterworth filter

# Filter specs

fp=1e3
fs=5e3
Ap=1
As=60

ws=2*np.pi*fs
wp=2*np.pi*fp

# Order and normaliz. freq. determination

N,Wn=signal.buttord(wp, ws, Ap, As, analog=True)

print ("Order = {0:d}".format(N))
print ("Normalizion frequency (Hz) {0:3g}".format(Wn/(2*np.pi)))

# Filter design with z p k output

z,p,k=signal.butter(N,Wn,"lowpass",analog=True,output="zpk")

# Poles and Zeroes are printed as module (w0) and Q
# instead of real and imaginary part
# The corresponding 2nd order polynomial is:
# s^2 + s*w0/Q + w0^2
# poles corresponds to denominator factors
# zeroes to numerator factors
# Complex poles and zeros are repeated twice
# Poles / zeroes that appears only once are real poles (Q should be ignored) 


print ("Poles \tModule (rad/s)\tQ")
print("\n")
for i in range(p.shape[0]):
    print ("\t"+str(abs(p[i]))+"\t"+str(-abs(p[i])/(2*np.real(p[i]))))
print("\n")
print ("Zeroes \tModule (rad/s)\tQ")
print("\n")
for i in range(z.shape[0]):
    if np.real(z[i])!=0:
        print ("\t"+str(abs(z[i]))+"\t"+str(-abs(z[i])/(2*np.real(z[i]))))
    else:
         print ("\t"+str(abs(z[i]))+"\t"+"Infinite (pure imag. zero)")

print("\n")
print("K ={0:5g}".format(k))
