import numpy as np
import scipy.signal as signal
import matplotlib.pyplot as plt

## This script illustrates how to use the basic function for analog
## time continuous filter design. The script is focused on low-pass
## filters

## These are the filter specifications
## Change them for your particular requirement

fp=1e3 # End of the pass-band (Hz)
fs=2e3 # Beginning of the stop band (Hz)
Ap=0.5 # Maximum attenuation in the pass-band (dB)
As=60  # Minimum attenuation in the stop-band (dB)

## Frequency to angular frequency transformation

ws=2*np.pi*fs
wp=2*np.pi*fp

# -----------------------------------------------------------------
# This is the step that determines the order (N) and normalization
# frequency (Wn , rad/s) of the filter
# Uncomment only the line corresponding to the filter of your choice

#N,Wn=signal.buttord(wp, ws, Ap, As, analog=True)
#N,Wn=signal.cheb1ord(wp, ws, Ap, As, analog=True)
N,Wn=signal.ellipord(wp, ws, Ap, As, analog=True)

print "Order",N
print "Normalization frequency",Wn/(2*np.pi), "Hz"

# ----------------------------------------------------------------
##This is the actual filter design
##The result are the denominator (a) and numerator (b) coefficients
##of the response in the laplace domain

##            b[0]s^m+...+ b[m-1]*z + b[m]
##    H(s) = ------------------------------
##             a[0]s^n+...+ a[n-1]*z + a[n]


#b,a=signal.butter(N,Wn,"lowpass",analog=True,output="ba")
#b,a=signal.cheb1(N,Ap,Wn,"lowpass",analog=True,output="ba")
b,a=signal.ellip(N,Ap,As,Wn,"lowpass",analog=True,output="ba")


# Frequency response 

w,mdb,p=signal.bode((b,a), n=10000)

p=p/180*np.pi
plt.semilogx(w/(2*np.pi),mdb)
plt.title("Magnitude (dB) response")
plt.show()

plt.semilogx(w/(2*np.pi),p)
plt.title("Phase response")
plt.show()

# Step response

t,y=signal.step((b,a))

plt.plot(t,y)
plt.title("Step response")
plt.show()

