import numpy as np
import scipy.signal as signal
import matplotlib.pyplot as plt

# This script illustrates the use of the function "iirdesign" for the
# direct synthesis of analog time-continuous filters
#
# This script demonstrates also the capability of the filter in
# separating two sinusoidal signal at different frequencies
# Use step2 function instead of step for stability reasons
# Use lsim2 function instead of lsim for stability reasons

# Filter Specifications

fp=1e3
fs=2e3
Ap=1
As=60


ws=2*np.pi*fs
wp=2*np.pi*fp




b,a=signal.iirdesign(wp, ws, Ap, As, analog=True, ftype='cheby1', output='ba')



w,mdb,p=signal.bode((b,a), n=10000)

#p=180/np.pi*np.unwrap(p/180*np.pi)
#p=np.unwrap(p/180*np.pi)
p=p/180*np.pi
plt.semilogx(w/(2*np.pi),mdb)
plt.title("Magnitude (dB) response")
plt.show()

plt.semilogx(w/(2*np.pi),p)
plt.title("Phase response")
plt.show()

t,y=signal.step2((b,a)) 

plt.plot(t,y)
plt.title("Step response")
plt.show()

# Creation of the two sinusoidal signals

t1=np.linspace(0,0.020,1000) # time axis

y1=np.sin(2*np.pi*t1*1000)  # first signal at 1 kHz

y1=y1+2*np.sin(2*np.pi*t1*2e3) # add a second signal at 2 kHz

# Uncomment the following line if you want to add random noise

y1=y1+0.5*(np.random.random(t1.shape)-0.5)

plt.plot(t1,y1)
plt.title("sum of 1 kH and 2 kHz sinusoids")
plt.show()

yf=signal.lsim2((b,a),y1,t1)
plt.plot(yf[0],yf[1],label="filtered")
plt.plot(t1,y1,label="unfiltered")
plt.legend()
plt.title("Effect of filter")
plt.show()
