import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
import scipy.signal as signal
import matplotlib as mpl

mpl.rc('xtick', labelsize=16)
mpl.rc('ytick', labelsize=16)
mpl.rc("axes",labelsize=18)


# Syntheis of first order DT systems starting from a CT system
# Bilinear transformation and simple integrator approximations are compared
# Note that the simple integrator - based approximation becomes
# unstable for f0 > fs/3.14 even if the CT system is stable.
# Note that fs/3.14 < fs/2, then is a valid singularity for the DT system
# For fs/(3.14)< f0 < fs/(2*3.14) the step response of the simple integrator
# is non monotonic (damped oscillations)


f0=5e3  # Frequency of the pole in Hz
fs=100e3 # Sampling frequency in Hz


w0=2*np.pi*f0
T=1.0/fs

# Time continuous filter: b=numerator(s), a=denominator(s)

a=np.array([1,w0]) # a[0]s + a[1]
b=np.array([w0,]) # b[0] 

w,p=signal.freqs(b,a,worN=np.logspace(-2,2,500)*w0) # Freq response
t,y=signal.step((b,a),T=np.linspace(0,20/w0,100)) # Step response

# DT synthesis using the simple integrator equivalence 1/s <--> T/(z-1)

bz2=np.array([0,w0*T])
az2=np.array([1,w0*T-1])
wz2,pz2=signal.freqz(bz2,az2) # Freq response
tz2,yz2=signal.dstep([bz2,az2,T],n=int(20/w0*fs)) # Step response

# Plots of the frequency response and impulse response

plt.loglog(w/2/np.pi,abs(p),label="CT")
plt.loglog(wz2/2/np.pi*fs,abs(pz2),label="SC")
plt.legend(loc=3)
plt.show()

plt.plot(t,y,label="CT")
plt.plot(tz2,yz2[0],label="SC")
plt.legend(loc=4)
plt.show()
